@@ -3,70 +3,87 @@ use warnings; use strict;
 
 =head1 NAME
 
-Elive - Elluminate Live! (c) client library
+Elive - Elluminate Live! (c) SDK bindings
 
 =head1 VERSION
 
-Version 0.53_1
+Version 0.74_2
 
 =cut
 
-our $VERSION = '0.53_1';
+our $VERSION = '0.74_2';
 
 use Class::Data::Inheritable;
 use base qw{Class::Data::Inheritable};
+use Scalar::Util;
 
 use YAML;
 
-=head1 SYNOPSIS
+use Carp;
+
+=head1 EXAMPLE
+
+The following (somewhat contrived) example sets up a meeting of selected
+participants:
 
     use Elive;
+    use Elive::Connection;
     use Elive::Entity::User;
+    use Elive::Entity::Meeting;
+
+    my $MeetingName = 'Meeting of the Smiths';
 
     Elive->connect('http://someEllumServer.com/test',
                    'serversupport', 'mypass');
 
-    my $users = Elive::Entity::User->list(filter => "(lastName = 'Smyth')");
+    my $participants = Elive::Entity::User->list(filter => "(lastName = 'Smith')");
+    die "smithless" unless @$participants;
 
-    foreach my $user (@$users) {
+    my $start = time() + 15 * 60; # starts in 15 minutes
+    my $end   = $start + 30 * 60; # runs for half an hour
 
-        printf ("changing last name for user: %s\n", $user->loginName);
+    my $meeting = Elive::Entity::Meeting->insert({
+	 name           => $MeetingName,
+	 facilitatorId  => Elive->login,
+	 start          => $start . '000',
+	 end            => $end   . '000',
+	 });
 
-        $user->lastName('Smith');
-        $user->update;
-
-    }
+    my $participant_list = $meeting->participant_list;
+    $participant_list->participants($participants);
+    $participant_list->update;
 
     Elive->disconnect;
 
 =head1 DESCRIPTION
 
-Elive is a set of Perl modules for the integration and automation of
-Elluminate I<Live!> sites. In particular, it aids in the management of users
-and meetings.
+Elive is a set of Perl bindings and entity definitions for the Elluminate
+I<Live!> SDK.
+
+The Elluminate SDK runs as a SOAP service and can be used to automate the
+raising, management and connection to meetings; and other related entities,
+including users, groups, preloads and recordings.
 
 =head1 BACKGROUND
 
-Elluminate I<Live!> (c) is is a web tool for virtual online classrooms.
+Elluminate I<Live!> (c) is software for virtual online classrooms.
 
-It is suitable for online collaboration, demonstrations, meetings, web
-conferences, seminars and IT deployment, training and support.
+It is suitable for meetings, demonstrations web conferences, seminars
+and IT deployment, training and support.
 
 Users, Meetings and other resources are stored in a management database.
-These can be accessed via the Elluminate I<Live!> SOAP API.
+These can be accessed and manipulated via the Elluminate I<Live!> SDK.
 
 Most actions that can be performed via the web interface can also be
-achieved via the SOAP API. This is known as the I<Command Toolkit> and
+achieved via the SOAP SDK. This is known as the I<Command Toolkit> and
 is detailed in chapter 4 of the Elluminate I<Live!> Software Developers
 Kit.
 
-This module provides Perl object to bindings to Elluminate I<Live!> entities
-via the command toolkit.
+This module provides Perl object bindings to Elluminate Live! entities via
+the Command Toolkit (SDK).
 
 =cut
 
-__PACKAGE__->mk_classdata('adapter' => 'default');
-
 our $DEBUG;
 BEGIN {
     $DEBUG = $ENV{ELIVE_DEBUG};
@@ -76,29 +93,28 @@ BEGIN {
 
 =head2 connect
 
-     my $e1 = Elive->connect('http://myServer.com/test1', user1, pass1);
+     my $e1 = Elive->connect('http://myServer.com/test1', 'user1', 'pass1');
 
-     Elive->connect('http://myServer.com/test2', user2, pass2);
+     Elive->connect('http://myServer.com/test2', 'user2', 'pass2');
      my $e2 = Elive->connection;
 
 Connects to an Elluminate server instance. Dies if the connection could not
 be established. If, for example, the SOAP connection or user login failed.
 
-The login user must be an Elluminate I<Live!> system administrator account.
+The login user must either be an Elluminate I<Live!> system administrator
+account, or a user that has been granted access to the SDK (see README file).
 
 See also Elive::Connection.
 
 =cut
 
 sub connect {
+    my ($class, $url, $login_name, $pass, %opts) = @_;
 
-    my $class = shift;
-    my ($url, $login_name, $pass) = @_;
+    die "usage: ${class}->new(url, [login_name] [, pass])"
+	unless ($class && $url);
 
-    die "usage: ${class}->new(url, login_name[, pass])"
-	unless ($class && $url && $login_name);
-
-    eval "use Elive::Connection";
+    eval {require Elive::Connection};
     die $@ if $@;
 
     my $connection = Elive::Connection->connect(
@@ -106,17 +122,11 @@ sub connect {
 	$login_name,
 	$pass,
 	debug => $class->debug,
+	%opts,
 	);
 
     $class->connection($connection);
 
-    #
-    # The login name should be a valid user in the database.
-    # retrieve it as a way of authenticating the user and
-    # checking basic connectivity.
-    #
-    $connection->login;
-
     return $connection;
 }
 
@@ -138,8 +148,7 @@ Returns the login user for the default connection.
 =cut
 
 sub login {
-    my $class = shift;
-    my %opt = @_;
+    my ($class, %opt) = @_;
 
     my $connection = $opt{connection} || $class->connection;
 
@@ -156,8 +165,7 @@ Returns the server details for the default connection.
 =cut
 
 sub server_details {
-    my $class = shift;
-    my %opt = @_;
+    my ($class, %opt) = @_;
 
     my $connection = $opt{connection} || $class->connection;
 
@@ -175,15 +183,14 @@ do this prior to exiting your program.
 =cut
 
 sub disconnect {
-    my $class = shift;
-    my %opt = @_;
+    my ($class, %opt) = @_;
 
-    if (my $connection = $opt{connection} || $class->connection) {
+    if (my $connection = $class->connection) {
 	$connection->disconnect;
 	$class->connection(undef);
     }
 
-    return undef;
+    return;
 }
 
 =head2 debug
@@ -200,128 +207,16 @@ sub disconnect {
 =cut
 
 sub debug {
-    my $class = shift;
+    my ($class, $level) = @_;
 
-    if (@_) {
-	$DEBUG = shift || 0;
+    if (defined $level) {
+	$DEBUG = $level;
     }
 
     return $DEBUG || 0;
 }
 
-our %KnownAdapters;
-
-BEGIN {
-    #
-    # classify adaptors as create, read, update or delete
-    #
-    %KnownAdapters = (
-	addGroupMember => 'c',
-	addMeetingPreload => 'c',
-	attendanceNotification => 'r',
-	changePassword => 'u',
-	buildMeetingJNLP => 'r',
-	buildRecordingJNLP => 'r',
-	checkMeetingPreload => 'r',
-	createGroup => 'c',
-	createMeeting => 'c',
-	createPreload => 'c',
-	createRecording => 'c',
-	createUser => 'c',
-	deleteGroup => 'd',
-	deleteMeeting => 'd',
-	deleteMeetingPreload => 'd',
-	deleteParticipant => 'd',
-	deleteRecording => 'd',
-	deletePreload => 'd',
-	deleteUser => 'd',
-	getGroup => 'r',
-	getMeeting => 'r',
-	getMeetingParameters => 'r',
-	getPreload => 'r',
-	getPreloadStream => 'r',
-	getRecording => 'r',
-	getRecordingStream => 'r',
-	getServerDetails => 'r',
-	getServerParameters => 'r',
-	getUser => 'r',
-	importPreload => 'c',
-	importRecording => 'c',
-	isParticipant => 'r',
-	listGroups => 'r',
-	listMeetingPreloads => 'r',
-	listMeetings => 'r',
-	listParticipants => 'r',
-	listPreloads => 'r',
-	listRecordings => 'r',
-	listUserMeetingsByDate => 'r',
-	listUsers => 'r',
-	resetGroup => 'u',
-	resetParticipantList => 'u',
-	setParticipantList => 'u',
-	streamPreload => 'u',
-	streamRecording => 'u',
-	updateMeeting => 'u',
-	updateMeetingParameters => 'u',
-	updateRecording => 'u',
-	updateServerParameters => 'u',
-	updateUser => 'u',
-	);
-}
-
-=head2 check_adapter
-
-    Elive->check_adapter('getUser')
-
-Asserts that the adapter is valid, i.e. it's in the list of known adapters.
-
-See also: elive_lint_config.
-
-=cut
-
-sub check_adapter {
-    my $class = shift;
-    my $adapter = shift;
-    my $crud = shift; #create, read, update or delete
-
-    my $usage = "usage: \$class->check_adapter(\$name[,'c'|'r'|'u'|'d'])";
-    die $usage unless $adapter;
-
-    my %known_adapters = $class->known_adapters;
-
-    die "Uknown adapter: $adapter"
-	unless exists $known_adapters{$adapter};
-
-    if ($crud) {
-	$crud = lc(substr($crud,0,1));
-	die $usage
-	    unless $crud =~ m{^[c|r|u|d]$};
-
-	my $adapter_type = $known_adapters{$adapter};
-	die "misconfigured adapter: $adapter"
-	    unless $adapter_type &&  $adapter_type  =~ m{^[c|r|u|d]$};
-
-	die "adapter $adapter. Type mismatch. Expected $crud, found $adapter_type"
-	    unless ($adapter_type eq $crud);
-    }
-
-    return $adapter;
-}
-
-=head2 known_adapters
-
-Returns an array of all Elluminate I<Live!> adapters required by Elive.
-This list is cross-checked by the script elive_lint_config. 
-
-=cut
-
-sub known_adapters {
-    my $class = shift;
-    return %KnownAdapters;
-}
-
 our %Meta_Data;
-our %Meta_Data_Accessor;
 
 =head2 has_metadata
 
@@ -334,11 +229,13 @@ sub has_metadata {
     my $class = shift;
     my $accessor = shift;
 
-    unless (exists $Meta_Data_Accessor{ $accessor }) {
+    my $accessor_fun = $class->can($accessor);
+
+    unless ($accessor_fun) {
 
 	no strict 'refs';
 
-	$Meta_Data_Accessor{ $accessor } ||= sub {
+	$accessor_fun = sub {
 	    my $self = shift;
 	    my $ref = $self->_refaddr
 		or return;
@@ -350,21 +247,16 @@ sub has_metadata {
 	    return $Meta_Data{ $ref }{ $accessor };
 	};
 
-	*{$class.'::'.$accessor} = $Meta_Data_Accessor{ $accessor }
+	*{$class.'::'.$accessor} = $accessor_fun;
     }
 
-    return $Meta_Data_Accessor{ $accessor };
+    return $accessor_fun;
 }
 
-sub DESTROY {
+sub DEMOLISH {
     my $self = shift;
-    delete  $Meta_Data{Scalar::Util::refaddr($self)};
-}
-
-{
-    no strict 'refs';
-
-    *{__PACKAGE__.'::DEMOLISH'} = \&DESTROY;
+    delete $Meta_Data{Scalar::Util::refaddr($self)};
+    return;
 }
 
 =head1 ERROR MESSAGES
@@ -373,75 +265,22 @@ Elluminate Services Errors:
 
 =over 4
 
-=item   "Unable to determine a command for the key : listXxxx"
+=item   "Unable to determine a command for the key : Xxxx"
 
-This may indicate that the particular command adaptor is is not available for
-your site instance.
+This may indicate that the particular command is is not available for your
+site instance. Please follow the instructions in the README file
+for detecting and repairing missing adapters.
 
-Check that your Elluminate I<Live!> server software version; This module
-has been tested against 9.0, 9.1 and 9.5.2
+=item   "User [<username>], not permitted to access the command {<command>]"
 
-If the problem persists, the command entry may be missing from your site
-configuration file. Please follow the instructions in the README file
-for instructions on detecting and repairing missing adapters.
+Please ensure that the user is a sytem administrator account and/or the
+user has been configured for SDK access. See also the instruction in the
+README file.
 
 =back
 
 =cut
 
-sub _check_for_errors {
-    my $class = shift;
-    my $som = shift;
-
-    die $som->fault->{ faultstring } if ($som->fault);
-
-    my $result = $som->result;
-    my @paramsout = $som->paramsout;
-
-    warn "result: ".YAML::Dump($result, @paramsout)
-	if ($class->debug);
-
-    if (@paramsout >= 2 && !$paramsout[1]) {
-	#
-	# error format sometimes seen with elluminate 9.6+. Can occur
-	# when request is malformed
-	#
-	die join(' ', $paramsout[0]);
-    }
-    elsif(!Elive::Util::_reftype($result)) {
-	#
-	# Simple scalar - we're done
-	#
-	return;
-    }
-    
-    #
-    # Look for Elluminate-specific errors
-    #
-    if (my $code = $result->{Code}{Value}) {
-
-	#
-	# Elluminate error!
-	#
-	
-	my $reason = $result->{Reason}{Text};
-
-	my $trace = $result->{Detail}{Stack}{Trace};
-	my @stacktrace;
-	if ($trace) {
-	    @stacktrace = (Elive::Util::_reftype($trace) eq 'ARRAY'
-			   ? @$trace
-			   : $trace);
-
-	}
-
-	my %seen;
-
-	my @error = grep {defined($_) && !$seen{$_}++} ($code, $reason, @stacktrace);
-	die join(' ', @error) || YAML::Dump($result);
-    }
-}
-
 =head1 SCRIPTS
 
 =head2 elive_query
@@ -450,7 +289,7 @@ elive_query is an example simple sql-like script. It is a basic program
 for listing and retrieving entities. It serves as a simple demonstration
 script, and can be used to confirm basic operation of Elive.
 
-It servers a secondary function of querying entity metadata. For example,
+It server a secondary function of querying entity metadata. For example,
 to show the user entity:
 
     $> elive_query
@@ -487,15 +326,13 @@ see the README file.
 
 =head1 SEE ALSO
 
-Perl Modules:
+L<Elive::API> - This is a separate CPAN module that implements the alternate Elluminate I<Live!> Standard Bridge API. 
+
+The following classes are included in this distribution and implement the SDK bindings:
 
 =over 4
 
-=item Elive::Connection - Elluminate SOAP connection
-
-=item Elive::Struct - base class for Elive::Entity
-
-=item Elive::Entity - base class for all elive entities
+=item Elive::Connection::SDK - Elluminate SOAP connection
 
 =item Elive::Entity::Group
 
@@ -509,6 +346,8 @@ Perl Modules:
 
 =item Elive::Entity::Recording
 
+=item Elive::Entity::Report
+
 =item Elive::Entity::ServerDetails
 
 =item Elive::Entity::ServerParameters
@@ -517,7 +356,7 @@ Perl Modules:
 
 =back
 
-Scripts:
+Scripts (included in the Elive distribution):
 
 =over 4
 
@@ -529,7 +368,7 @@ Scripts:
 
 =back
 
-Elluminate I<Live!> Documentation. This comes with your distribution
+Elluminate I<Live!> Documentation, installed with Elluminate.
 
 =over 4
 
@@ -585,17 +424,20 @@ David Warring, C<< <david.warring at gmail.com> >>
 
 =item Elive is a newish module
 
-I have so far run it against a limited number of Elluminate 9.0, 9.1
-and 9.5 installations.
+It has been used and tested against a number of sites running Elluminate 9.0
+to 10.0.1.
 
 So far it does not implement all SOAP calls, but concentrates on entities
 such as users, meetings, preloads and meeting participants.
 
 =item Elive does not support hosted (SAS) systems
 
-The package currently supports only the installed server version of Elluminate
-Live which uses the ELM management layer. It does not yet support the hosted
-servers deployed with SAS (Session Administration System).
+The Elive distribution only supports the Elluminate SDK which is implemented
+by ELM (Elluminate Live Manager) session manager. This SDK is not support by
+SAS (Session Administration System).
+
+Update: See also the companion CPAN module L<Elive::API> which is under
+construction and will support the alternate Elluminate I<Live!> standard bridge.
 
 =back
 
@@ -634,12 +476,12 @@ L<http://search.cpan.org/dist/Elive/>
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to Lex Lucas and Simon Haidley for their testing, support and
-direction during the construction of this module.
+Thanks to Lex Lucas and Simon Haidley for their ongoing support and
+assistance with the development of this module.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 David Warring, all rights reserved.
+Copyright 2009-2010 David Warring, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
