@@ -3,14 +3,14 @@ use warnings; use strict;
 
 use Mouse;
 
-use Elive::Entity;
-use base qw{ Elive::Entity };
+extends 'Elive::Entity';
 
 use Elive::Util;
 use Elive::Entity::Preload;
 use Elive::Entity::Recording;
 use Elive::Entity::MeetingParameters;
 use Elive::Entity::ServerParameters;
+use Elive::Entity::ParticipantList;
 
 use YAML;
 
@@ -29,6 +29,16 @@ Elive::Entity::MeetingParameters and Elive::Entity::ServerParameters.
 
 __PACKAGE__->entity_name('Meeting');
 __PACKAGE__->collection_name('Meetings');
+__PACKAGE__->params(
+    seats => 'Int',
+    preloadId => 'Int',
+    recurrenceCount => 'Int',
+    recurrenceDays => 'Int',
+    timeZone => 'Str',
+    userId => 'Str',
+    startDate => 'HiResDate',
+    endDate => 'HiResDate',
+    );
 
 has 'meetingId' => (is => 'rw', isa => 'Int', required => 1);
 __PACKAGE__->primary_key('meetingId');
@@ -62,6 +72,8 @@ has  'allModerators' => (is => 'rw', isa => 'Bool',
 has  'restrictedMeeting' => (is => 'rw', isa => 'Bool',
 			     documentation => "Restricted meeting");
 
+has 'adapter' => (is => 'rw', isa => 'Str',
+		  documentation => 'adapter used to create the meeting/session');
 
 =head1 METHODS
 
@@ -69,95 +81,55 @@ has  'restrictedMeeting' => (is => 'rw', isa => 'Bool',
 
 =head2 insert
 
-=head3 synopsis
+    my $start = time() + 15 * 60; # starts in 15 minutes
+    my $end   = $start + 30 * 60; # runs for half an hour
 
-    #
-    # Simple case, single meeting
-    #
     my $meeting = Elive::Entity::Meeting->insert({
-        start => time,
-        end => time,
-        name => string,
-        password => string,
-        seats => int,
-        privateMeeting => 0|1,
-        timeZone => string
-       });
+	 name              => 'Test Meeting',
+	 facilitatorId     => Elive->login,
+	 start             => $start . '000',
+	 end               => $end   . '000',
+         password          => 'secret!',
+         privateMeeting    => 1,
+         restrictedMeeting => 1,
+         seats             => 42,
+	 });
 
     #
-    # A recurring series of meetings:
+    # Set the meeting participants
+    #
+    my $participant_list = $meeting->participant_list;
+    $participant_list->participants([qw(smith jones)]);
+    $participant_list->update;
+
+A series of meetings can be created using the C<recurrenceCount> and
+C<recurrenceDays> parameters.
+
+    #
+    # create three weekly meetings
     #
     my @meetings = Elive::Entity::Meeting->insert({
                             ...,
-                            recurrenceCount => n,
-                            recurrenceDays => 7,
+                            recurrenceCount => 3,
+                            recurrenceDays  => 7,
                         });
-
 =cut
 
-sub insert {
-    my $class = shift;
-    my $data = shift;
-
-    die "usage: $class->insert(\\%data, %opts)"
-	unless (Elive::Util::_reftype($data) eq 'HASH'
-		&& @_ % 2 == 0);
- 
-    my %opt = @_;
-
-    my %params = (seats => 'Int',
-		  recurrenceCount => 'Int',
-		  recurrenceDays => 'Int',
-		  timeZone => 'Str');
-
-    foreach (keys %params) {
-	my $type = $params{$_};
-	#
-	# these are parameters, not properties
-	#
-	$opt{param}{$_} = Elive::Util::_freeze(delete $data->{$_}, $type)
-	    if exists $data->{$_}
-    }
-
-    $class->SUPER::insert($data, %opt);
-}
-
 =head2 update
 
-=head3 synopsis
-
     my $meeting = Elive::Entity::Meeting->update({
-        start => time,
-        end => time,
-        name => string,
-        password => string,
-        seats => int,
-        privateMeeting => 0|1,
-        timeZone => string
+        start             => hires-date,
+        end               => hires-date,
+        name              => string,
+        password          => string,
+        seats             => int,
+        privateMeeting    => 0|1,
+        restrictedMeeting => 0|1,
+        timeZone          => string
        });
 
 =cut
 
-sub update {
-    my $self = shift;
-    my $data = shift;
-    my %opt = @_;
-
-    my %params = (seats => 'Int',
-		  timeZone => 'Str');
-
-    foreach (keys %params) {
-	my $type = $params{$_};
-	#
-	# these are parameters, not properties
-	#
-	$opt{param}{$_} = Elive::Util::_freeze(delete $data->{$_}, $type)
-	    if exists $data->{$_}
-    }
-
-    $self->SUPER::update($data, %opt);
-}
-
 =head2 delete
 
     my $meeting = Elive::Entity::Meeting->retrieve([$meeting_id]);
@@ -165,69 +137,84 @@ sub update {
 
 Delete the meeting.
 
-Note: With Elluminate 9.5 onwards, deleting a meetings simply sets the
-I<deleted> property to true. Meetings, Meeting Parameters and Server
-Parameters remain accessable via the SOAP inteface.
+Note:
 
-If you only want live meetings, you'll need to check for deleted meetings:
+=over 4
+
+=item Meeting recordings are not deleted.
+
+If you also want to remove the associated recordings, you'll need to delete
+them yourself, E.g.:
+
+    my $recordings = $meeting->list_recordings;
+
+    foreach my $recording (@$recordings) {
+        $recording->delete;
+    }
+
+    $meeting->delete;
+
+=item With Elluminate 9.5 onwards simply sets the I<deleted> property to true.
+
+Meetings, Meeting Parameters, Server Parameters and recordings remain
+accessable via the SOAP inteface.
+
+You'll need to remember to check for deleted meetings:
 
     my $meeting =  Elive::Entity::Meeting->retrieve([$meeting_id]);
     my $is_live = $meeting->deleted;
 
-or filter out deleted meetings:
+or filter them out when listing meetings:
 
     my $live_meetings =  Elive::Entity::Meeting->list(filter => 'deleted = false');
 
+=back
+
 =cut
 
 =head2 list_user_meetings_by_date
 
-List all meetings over a given date range.
+Lists all meetings for which this user is a participant, over a given
+date range.
 
-=head3 synopsis
-
-   $meetings_array_ref
-           =  Elive::Entity::Meeting->list_user_meetings_by_date(
-                         [$user_obj_or_id,
-                          $hires_start_date,
-                          $hires_end_date]
-                      );
-
-=head3 example
+For example, to list all meetings for a particular user over the next week:
 
    my $now = DateTime->now;
    my $next_week = $now->clone->add(days => 7);
 
    my $meetings = Elive::Entity::Meeting->list_user_meetings_by_date(
-    [$user_id, $now->epoch.'000', $next_week->epoch.'000']
-  )
+        [$user_id, $now->epoch.'000', $next_week->epoch.'000']
+       );
 
-=head3 description
+   # equivalently:
 
-Lists all meetings for which this user is a participant.
-
-Implements the ListUserMeetingsByDateCommand SDK method.
+   my $meetings = Elive::Entity::Meeting->list_user_meetings_by_date(
+        {userId => $user_id,
+         startDate => $now->epoch.'000',
+         endDate => $next_week->epoch.'000'}
+       );
 
 =cut
 
 sub list_user_meetings_by_date {
-    my $class = shift;
-    my $params = shift;
-    my %opt = @_;
-
-    die 'usage: $class->user_meetings_by_date([$user, $start_date, $end_date])'
-	unless (Elive::Util::_reftype($params) eq 'ARRAY'
-		&& $params->[0] && @$params <= 3);
+    my ($class, $params, %opt) = @_;
 
     my %fetch_params;
-    $fetch_params{userId}    = Elive::Util::_freeze(shift @$params,'Str');
-    $fetch_params{startDate} = Elive::Util::_freeze(shift @$params,'HiResDate');
-    $fetch_params{endDate} = Elive::Util::_freeze(shift @$params,'HiResDate');
+    my $reftype = Elive::Util::_reftype($params);
 
-    my $adapter = $class->check_adapter('listUserMeetingsByDate');
+    if ($reftype eq 'HASH') {
+	%fetch_params = %$params;
+    }
+    elsif ($reftype eq 'ARRAY'
+	   && $params->[0] && @$params <= 3) {
+	@fetch_params{qw{userId startDate endDate}} = @$params;
+    }
+    else {
+	die 'usage: $class->user_meetings_by_date([$user, $start_date, $end_date])'
+    }
 
-    return $class->_fetch(\%fetch_params,
-			  adapter => $adapter,
+    return $class->_fetch($class->_freeze(\%fetch_params),
+			  command => 'listUserMeetingsByDate',
 			  %opt,
 	);
 }
@@ -237,7 +224,8 @@ sub list_user_meetings_by_date {
     my $meeting = Elive::Entity::Meeting->retrieve($meeting_id);
     $meeting->add_preload($preload_id);
 
-Associate a preload with a meeting
+Associates a preload with a meeting. This preload must pre-exist in the
+database.
 
 =head3 See also
 
@@ -246,40 +234,27 @@ Elive::Entity::Preload
 =cut
 
 sub add_preload {
-    my $self = shift;
-    my $preload = shift;
-    my %opt = @_;
+    my ($self, $preload_id, %opt) = @_;
 
     die 'usage: $meeting_obj->add_preload($preload || $preload_id)'
-	unless $preload;
+	unless $preload_id;
 
-    my $meeting_id = $opt{meeting_id};
-
-    $meeting_id ||= $self->meetingId
-	if ref($self);
+    my %params = %{ $opt{param} || {} };
 
+    my $meeting_id = $opt{meeting_id} || $self->meetingId;
     die "unable to determine meeting_id"
 	unless $meeting_id;
 
-    my $preload_id = ref($preload)
-	? $preload->preloadId
-	: $preload;
-
-    die "unable to determine preload_id"
-	unless $preload_id;
-
-    my $adapter = $self->check_adapter('addMeetingPreload');
+    $params{meetingId} ||= $meeting_id;
+    $params{preloadId} = $preload_id;
 
     my $connection = $self->connection
 	or die "not connected";
 
-    my $som = $connection
-	->call($adapter,
-	       meetingId => Elive::Util::_freeze($meeting_id, 'Int'),
-	       preloadId => Elive::Util::_freeze($preload_id, 'Int'),
-	);
+    my $som = $connection->call('addMeetingPreload',
+				%{ $self->_freeze( \%params ) });
 
-    $self->_check_for_errors($som);
+    return $connection->_check_for_errors($som);
 }
 
 =head2 check_preload
@@ -291,25 +266,26 @@ Checks that the preload is associated with this meeting.
 =cut
 
 sub check_preload {
-    my $self = shift;
-    my $preload_id = shift;
-    my %opt = @_;
+    my ($self, $preload_id, %opt) = @_;
 
-    $preload_id = $preload_id->preloadId
-	if ref($preload_id);
+    die 'usage: $meeting_obj->check_preload($preload || $preload_id)'
+	unless $preload_id;
 
-    my $adapter = $self->check_adapter('checkMeetingPreload');
+    my $meeting_id = $opt{meeting_id} || $self->meetingId;
+
+    die "unable to determine meeting_id"
+	unless $meeting_id;
 
     my $connection = $self->connection
 	or die "not connected";
 
     my $som = $connection
-	->call($adapter,
+	->call('checkMeetingPreload',
 	       preloadId => Elive::Util::_freeze($preload_id, 'Int'),
-	       meetingId => Elive::Util::_freeze($self->meetingId, 'Int'),
+	       meetingId => Elive::Util::_freeze($meeting_id, 'Int'),
 	       );
 
-    $self->_check_for_errors($som);
+    $connection->_check_for_errors($som);
 
     my $results = $self->_unpack_as_list($som->result);
 
@@ -325,35 +301,51 @@ Checks that the user is a meeting participant.
 =cut
 
 sub is_participant {
-    my $self = shift;
-    my $user_id = shift;
-    my %opt = @_;
+    my ($self, $user, %opt) = @_;
 
-    $user_id = $user_id->userId
-        if ref($user_id);
+    die 'usage: $meeting_obj->is_preload($user || $user_id)'
+	unless $user;
 
-    my $adapter = $self->check_adapter('isParticipant');
+    my $meeting_id = $opt{meeting_id} || $self->meetingId;
+
+    die "unable to determine meeting_id"
+	unless $meeting_id;
 
     my $connection = $self->connection
         or die "not connected";
 
+    my $command = $opt{command} || 'isParticipant';
+
     my $som = $connection
-        ->call($adapter,
-               userId => Elive::Util::_freeze($user_id, 'Str'),
-               meetingId => Elive::Util::_freeze($self->meetingId, 'Int'),
+        ->call($command,
+               userId => Elive::Util::_freeze($user, 'Str'),
+               meetingId => Elive::Util::_freeze($meeting_id, 'Int'),
                );
 
-    $self->_check_for_errors($som);
+    $connection->_check_for_errors($som);
 
     my $results = $self->_unpack_as_list($som->result);
 
     return @$results && Elive::Util::_thaw($results->[0], 'Bool');
 }
 
+=head2 is_moderator
+
+    my $ok = $meeting_obj->is_moderator($user);
+
+Checks that the user is a meeting moderator.
+
+=cut
+
+sub is_moderator {
+    my ($self, $user, %opt) = @_;
+
+    return $self->is_participant($user, %opt, command => 'isModerator');
+}
+
 sub _readback_check {
-    my $class = shift;
-    my %updates = %{shift()};
-    my $rows = shift;
+    my ($class, $updates_href, $rows, @args) = @_;
+    my %updates = %$updates_href;
 
     #
     # password not included in readback record - skip it
@@ -367,90 +359,74 @@ sub _readback_check {
     #
     $rows = [$rows->[0]] if @$rows > 1;
 
-    $class->SUPER::_readback_check(\%updates, $rows, @_);
-}
-
-sub _thaw {
-    my $class = shift;
-    my $db_data = shift;
-
-    my $data = $class->SUPER::_thaw($db_data, @_);
-
-    if ($data->{Adapter}) {
-	#
-	# meeting giving us unexpected Adapter property. Ignore it
-	#
-
-	if ($class->debug) {
-	    print STDERR "Stray meeting adaptor found:\n";
-	    print STDERR YAML::Dump($data->{Adapter});
-	    print STDERR "\n";
-	}
-	
-	delete $data->{Adapter};
-    }
-
-    return $data;
+    return $class->SUPER::_readback_check(\%updates, $rows, @args);
 }
 
 =head2 remove_preload
 
-    $meeting_obj->remove_preload($preload_id_or_obj);
+    $meeting_obj->remove_preload($preload_obj);
+    $preload_obj->delete;  # if you don't want it to hang around
 
 Disassociate a preload from a meeting.
 
-Note that the preload object is not actually deleted, just disassociated
-from the meeting and will continue to exist as a resource on the server.
-
-You don't need to call this method prior to deleting the preload. The
-deletion will succeed and the preload is disassociated from all meetings.
-
 =cut
 
 sub remove_preload {
-    my $self = shift;
-    my $preload_id = shift;
-    my %opt = @_;
+    my ($self, $preload_id, %opt) = @_;
 
-    my $meeting_id = $self->meetingId;
+    my $meeting_id = $opt{meeting_id} || $self->meetingId;
 
     die 'unable to get a meeting_id'
 	unless $meeting_id;
 
-    $preload_id = $preload_id->preloadId
-	if ref($preload_id);
-
-    die 'unable to get a preload_id'
+    die 'unable to get a preload'
 	unless $preload_id;
 
     my $connection = $self->connection
 	or die "not connected";
 
-    my $adapter = $self->check_adapter('deleteMeetingPreload');
-
-    my $som = $connection->call($adapter,
+    my $som = $connection->call('deleteMeetingPreload',
 				meetingId => Elive::Util::_freeze($meeting_id, 'Int'),
 				preloadId => Elive::Util::_freeze($preload_id, 'Int'),
 				);
 
-    $self->_check_for_errors($som);
+    return $connection->_check_for_errors($som);
 }
     
 =head2 buildJNLP 
 
+    # ...
+    use Elive;
+    use Elive::Entity::Meeting;
+
+    use CGI;
+    my $cgi = CGI->new;
+
+    #
+    # [authentication etc goes here] ...
+    #
+
     my $jnlp = $meeting_entity->buildJNLP(version => $version,
 					  user => $userId||$userName,
 					  pass => $password,
-                                          displayName => 'Display Name'
-    );
+                                          displayName => $displayName,
+                                         );
+    #
+    # join this user to the meeting
+    #
+
+    print $cgi->header(-type       => 'application/x-java-jnlp-file',
+                       -attachment => 'my-meeting.jnlp');
+
+    print $jnlp;
 
 Builds a JNLP for the meeting.
 
 JNLP is the 'Java Network Launch Protocol', also commonly known as Java
-WebStart. You can, for example, render this as a web page with mime type
-C<application/x-java-jnlp-file>.
+WebStart. To launch the meeting you can, for example, render this as a web
+page, or send email attachments  with mime type C<application/x-java-jnlp-file>.
 
-Under Windows, and other desktops, you can save this to a file with extension
+Under Windows, and other desktops, files are usually saved  with extension
 C<JNLP>.
 
 See also L<http://en.wikipedia.org/wiki/JNLP>.
@@ -458,16 +434,12 @@ See also L<http://en.wikipedia.org/wiki/JNLP>.
 =cut
 
 sub buildJNLP {
-    my $self = shift;
-    my %opt = @_;
+    my ($self, %opt) = @_;
 
-    my $connection = $self->connection
+    my $connection = $self->connection || $opt{connection}
 	or die "not connected";
 
-    my $meeting_id = $opt{meeting_id};
-
-    $meeting_id ||= $self->meetingId
-	if ref($self);
+    my $meeting_id = $opt{meeting_id} ||= $self->meetingId;
 
     die "unable to determine meeting_id"
 	unless $meeting_id;
@@ -482,20 +454,14 @@ sub buildJNLP {
 
     for (delete $opt{user} || $connection->login->userId) {
 
-	$soap_params{m{^\d+$}? 'userId' : 'userName'} = Elive::Util::_freeze($_, 'Str');
+	$soap_params{m{^\d+$}x? 'userId' : 'userName'} = Elive::Util::_freeze($_, 'Str');
     }
 
-    my $adapter = $self->check_adapter('buildMeetingJNLP');
+    my $som = $connection->call('buildMeetingJNLP', %soap_params);
 
-    my $som = $connection->call($adapter,
-				%soap_params,
-				);
+    my $results = $self->_get_results($som, $connection);
 
-    $self->_check_for_errors($som);
-
-    my $results = $self->_unpack_as_list($som->result);
-
-    return @$results && Elive::Util::_thaw($results->[0], 'Str');
+    return @$results && $results->[0];
 }
 
 =head2 web_url
@@ -503,8 +469,6 @@ sub buildJNLP {
 Utility method to return various website links for the meeting. This is
 available as both class level and object level methods.
 
-=head3 Examples
-
     #
     # Class level access.
     #
@@ -522,10 +486,14 @@ available as both class level and object level methods.
 =cut
 
 sub web_url {
-    my $self = shift;
-    my %opt = @_;
+    my ($self, %opt) = @_;
+
+    my $meeting_id = $opt{meeting_id} || $self->meetingId;
+    $meeting_id = Elive::Util::_freeze($meeting_id, 'Str');
+
+   die "no meeting_id given"
+	unless $meeting_id;
 
-    my $meeting_id = $opt{meeting_id};
     my $connection = $self->connection || $opt{connection}
 	or die "not connected";
 
@@ -570,13 +538,13 @@ See also L<Elive::Entity::MeetingParameters>.
 =cut
 
 sub parameters {
-    my $self = shift;
+    my ($self, @args) = @_;
 
     return Elive::Entity::MeetingParameters
 	->retrieve([$self->meetingId],
 		   reuse => 1,
 		   connection => $self->connection,
-		   @_,
+		   @args,
 	);
 }
 
@@ -591,13 +559,13 @@ See also L<Elive::Entity::ServerParameters>.
 =cut
 
 sub server_parameters {
-    my $self = shift;
+    my ($self, @args) = @_;
 
     return Elive::Entity::ServerParameters
 	->retrieve([$self->meetingId],
 		   reuse => 1,
 		   connection => $self->connection,
-		   @_,
+		   @args,
 	);
 }
 
@@ -612,13 +580,13 @@ See also L<Elive::Entity::ParticipantList>.
 =cut
 
 sub participant_list {
-    my $self = shift;
+    my ($self, @args) = @_;
 
     return Elive::Entity::ParticipantList
 	->retrieve([$self->meetingId],
 		   reuse => 1,
 		   connection => $self->connection,
-		   @_,
+		   @args,
 	);
 }
 
@@ -631,12 +599,12 @@ Lists all preloads associated with the meeting. See also L<Elive::Entity::Preloa
 =cut
 
 sub list_preloads {
-    my $self = shift;
+    my ($self, @args) = @_;
 
     return Elive::Entity::Preload
         ->list_meeting_preloads($self->meetingId,
 				connection => $self->connection,
-				@_);
+				@args);
 }
 
 =head2 list_recordings
@@ -649,14 +617,28 @@ L<Elive::Entity::Recording>.
 =cut
 
 sub list_recordings {
-    my $self = shift;
+    my ($self, @args) = shift;
 
     return Elive::Entity::Recording
 	->list(filter => 'meetingId = '.$self->meetingId,
 	       connection => $self->connection,
-	       @_);
+	       @args);
 }
-    
+
+=head1 BUGS AND LIMITATIONS
+
+=over 4
+
+=item
+
+Meetings can not be set to restricted (as of Elluminate 9.7 - 10.0), nor
+does the SDK respect the server default settings for restricted meetings.
+
+Update: As of Elluminate 10.0.1, the restrictedMeeting property is inherited
+from Preferences E<gt>E<gt> Session Defaults E<gt>E<gt> Restrict Meetings.
+
+=back
+
 =head1 SEE ALSO
 
 =over 4
@@ -669,6 +651,8 @@ sub list_recordings {
 
 =item Elive::Entity::ServerParameters
 
+=item Elive::Entity::ParticipantList
+
 =back
 
 =cut
