@@ -4,10 +4,10 @@ use warnings; use strict;
 use Mouse;
 use Mouse::Util::TypeConstraints;
 
-use Elive::Entity;
-use base qw{ Elive::Entity };
+extends 'Elive::Entity';
 
 __PACKAGE__->entity_name('MeetingParameters');
+__PACKAGE__->_isa('Meeting');
 
 has 'meetingId' => (is => 'rw', isa => 'Int', required => 1,
     documentation => 'associated meeting');
@@ -45,15 +45,26 @@ has 'profile'  => (is => 'rw', isa => 'Str');
 
 Elive::Entity::MeetingParameters - Meeting parameters entity class
 
+    use Elive::Entity::MeetingParameters;
+
     my $meeting_params
         = Elive::Entity::MeetingParameters->retrieve([$meeting_id]);
 
-    $meeting_params->maxTalkers(5);
-    $meeting_params->update;
+    $meeting_params->update({
+           maxTalkers => 3,
+           costCenter => 'acme',
+           moderatorNotes => 'be there early!',
+           userNotes => "don't be late!",
+           recordingStatus => 'on',
+           raiseHandsOnEnter => 1,
+           inSessionInvitation => 1,
+           followModerator => 0,
+           videoWindow => 0,
+         });
 
 =head1 DESCRIPTION
 
-This class contains additional meeting information.
+This class contains a range of options for a previously created meeting.
 
 =cut
 
@@ -68,7 +79,7 @@ automatically created when you create a table.
 
 =cut
 
-sub insert {shift->_not_available}
+sub insert {return shift->_not_available}
 
 =head2 delete
 
@@ -77,7 +88,7 @@ when the meeting itself is deleted.
 
 =cut
 
-sub delete {shift->_not_available}
+sub delete {return shift->_not_available}
 
 =head2 list
 
@@ -86,20 +97,19 @@ to create a meeting, then retrieve on meeting id
 
 =cut
 
-sub list {shift->_not_available}
+sub list {return shift->_not_available}
 
 sub _thaw {
-    my $class = shift;
-    my $db_data = shift;
+    my ($class, $db_data, @args) = @_;
 
-    my $data = $class->SUPER::_thaw($db_data, @_);
+    my $data = $class->SUPER::_thaw($db_data, @args);
 
     for (grep {defined} $data->{recordingStatus}) {
 
 	$_ = lc($_);
 
-	unless (m{^(on|off|remote)$} || $_ eq '') {
-	    warn "ignoring unknown recording status: $_";
+	unless (m{^(on|off|remote)$}x || $_ eq '') {
+	    warn "ignoring unknown recording status: $_\n";
 	    delete  $data->{recordingStatus};
 	}
     }
