@@ -4,17 +4,18 @@ use warnings; use strict;
 use Mouse;
 use Mouse::Util::TypeConstraints;
 
-use Elive::Entity;
-use base qw{ Elive::Entity };
+extends 'Elive::Entity';
 
 use Elive::Entity::Role;
+use Elive::Util;
 
 __PACKAGE__->entity_name('User');
 __PACKAGE__->collection_name('Users');
 
 has 'userId' => (is => 'rw', isa => 'Str', required => 1,
-		 documentation => 'user identifier (usually numeric)');
+		 documentation => 'user identifier (numeric, unless LDAP configured)');
 __PACKAGE__->primary_key('userId');
+__PACKAGE__->params(userName => 'Str');
 
 has 'deleted' => (is => 'rw', isa => 'Bool');
 
@@ -22,6 +23,7 @@ has 'loginPassword' => (is => 'rw', isa => 'Str');
 
 has 'loginName' => (is => 'rw', isa => 'Str',
 		    documentation => 'login name - must be unique');
+__PACKAGE__->_alias(userName => 'loginName');
 		    
 has 'email' => (is => 'rw', isa => 'Str',
 		documentation => 'users email address');
@@ -42,10 +44,10 @@ has 'lastName' => (is => 'rw', isa => 'Str',
 #
 
 has 'groups' => (is => 'rw', isa => 'Str',
-		documentation => 'groups?');
+		documentation => 'ldap groups?');
 
 has 'domain' => (is => 'rw', isa => 'Str',
-		 documentation => 'domain?');
+		 documentation => 'ldap domain?');
 
 coerce 'Elive::Entity::User' => from 'HashRef'
           => via {Elive::Entity::User->construct($_,
@@ -66,8 +68,9 @@ coerce 'Elive::Entity::User' => from 'Str'
 =cut
 
 sub _readback_check {
-    my $class = shift;
-    my %updates = %{shift()};
+    my ($class, $update_ref, $rows, @args) = @_;
+
+    my %updates = %$update_ref;
 
     #
     # password not included in readback record - skip it
@@ -75,7 +78,7 @@ sub _readback_check {
 
     delete $updates{loginPassword};
 
-    $class->SUPER::_readback_check(\%updates, @_, case_insensitive => 1);
+    return $class->SUPER::_readback_check(\%updates, $rows, @args, case_insensitive => 1);
 }
 
 =head2 get_by_loginName
@@ -84,17 +87,20 @@ sub _readback_check {
 
 Retrieve on loginName, which is a co-key for the users table.
 
+Please note that the Elluminate Web Services raise an error if the user
+was not found. So, if you're not sure if the user exists:
+
+    my $user = eval {Elive::Entity::User->get_by_loginName('joebloggs')};
+
 =cut
 
 sub get_by_loginName {
-    my $class = shift;
-    my $loginName = shift;
+    my ($class, $loginName, @args) = @_;
     #
     # The entity name is loginName, but the fetch key is userName.
     #
     my $results = $class->_fetch({userName => $loginName},
-				 readback => {loginName => $loginName},
-				 @_,
+				 @args,
 	);
 
     return @$results && $results->[0];
@@ -116,16 +122,14 @@ Insert a new user
 =cut
 
 sub insert {
-    my $class = shift;
-    my %data = %{shift()};
-    my %opt = @_;
+    my ($class, $data_href, %opt) = @_;
 
-    my $self = $class->SUPER::insert( \%data, %opt );
+    my $self = $class->SUPER::insert( $data_href, %opt );
 
     #
     # seems we have to insert a record, then set the password
     #
-    my $password = $data{loginPassword};
+    my $password = $data_href->{loginPassword};
     if (defined $password and $password ne '') {
 	$self->change_password($password);
     }
@@ -146,15 +150,15 @@ sub insert {
 Update an Elluminate user. Everything can be changed, other than userId.
 This includes the loginName. However loginNames must all remain unique.
 
-As a safeguard, you need to pass force => 1 to update users with a Role
-Id of 0, i.e. system administrator accounts
+As a safeguard, you'll need to pass C<force =E<gt> 1> to update:
+    (a) users with a Role Id of 0, i.e. system administrator accounts, or
+    (b) the login user
 
 =cut
 
 sub update {
-    my $self = shift;
-    my %update_data = %{shift() || {}};
-    my %opt = @_;
+    my ($self, $data_href, %opt) = @_;
+    my %update_data = %{$data_href || {}};
 
     unless ($opt{force}) {
 	die "Cowardly refusing to update system admin account for ".$self->loginName.": (pass force => 1 to override)"
@@ -180,29 +184,28 @@ sub update {
     return $ret;
 }
 
-=head2 change_password 
+=head2 change_password
+
+Implements the C<changePassword> SDK method.
 
     my $user = Elive::Entity::User->retrieve([$user_id]);
-    my $new_password = Elive::Util::prompt('Password: ', password => 1);
     $user->change_password($new_password);
 
+This is equivalent to:
+
+    my $user = Elive::Entity::User->retrieve([$user_id]);
+    $user->update({loginPassword => $new_password});    
+
 =cut
 
 sub change_password {
-    my $self = shift;
-    my $new_password = shift;
-    my %opt = @_;
+    my ($self, $new_password, %opt) = @_;
 
     $self->SUPER::update({loginPassword => $new_password},
 			 adapter => 'changePassword',
 			 %opt,
 	)
 	if (defined $new_password && $new_password ne '');
-    #
-    # Revert to the readback copy. This does not contain a copy of
-    # the password. 
-    #
-    $self->revert;
 
     return $self;
 }
@@ -211,18 +214,27 @@ sub change_password {
 
     $user_obj->delete([force => 1]);
 
-Delete user objects. As a safeguard, you need to pass force => 1 to update
-system administrator accounts
+Delete user objects. As a safeguard, you need to pass C<force =E<gt> 1> to delete
+system administrator accounts, or the login user.
 
 =cut
 
 sub delete {
-    my $self = shift;
-    my %opt = @_;
+    my ($self, %opt) = @_;
 
     unless ($opt{force}) {
 	die "Cowardly refusing to delete system admin account for ".$self->loginName.": (pass force => 1 to override)"
-	    if ($self->role <= 0);
+	    if (Elive::Util::string($self->role) == 0);
+
+	my $connection = $self->connection;
+
+	my $login = $connection->login;
+	die "Not loggged in" unless $login;
+	#
+	# Less cowardly, methinks!
+	#
+	die "Refusing to delete the login user ".$login->loginName.": (pass force => 1 to override)"
+	    if $login->userId eq $self->userId;   
     }
 
     return $self->SUPER::delete( %opt );
