@@ -1,28 +1,17 @@
 package Elive::Struct;
 use warnings; use strict;
 
+use Mouse;
 use Elive;
 use base qw{Elive};
 
 use Elive::Array;
 use Elive::Util;
 
+use Carp;
+
 use YAML;
 
-BEGIN {
-    __PACKAGE__->mk_classdata('_entities' => {});
-    __PACKAGE__->mk_classdata('_aliases');
-    __PACKAGE__->mk_classdata('_entity_name');
-    __PACKAGE__->mk_classdata('_primary_key', []);
-    __PACKAGE__->mk_classdata('collection_name');
-};
-
-use Scalar::Util;
-
-sub _refaddr {
-    return Scalar::Util::refaddr( shift() );
-}
-
 =head1 NAME
 
 Elive::Struct - Base class for Elive::Entity and entity sub-structures
@@ -64,6 +53,75 @@ In particular meeting participants stringify to userId=role, eg
 
 =cut
 
+BEGIN {
+    __PACKAGE__->mk_classdata('_entities' => {});
+    __PACKAGE__->mk_classdata('_aliases');
+    __PACKAGE__->mk_classdata('_entity_name');
+    __PACKAGE__->mk_classdata('_primary_key', []);
+    __PACKAGE__->mk_classdata('_params', {});
+    __PACKAGE__->mk_classdata('collection_name');
+    __PACKAGE__->mk_classdata('_isa');
+};
+
+use Scalar::Util;
+
+sub _refaddr {
+    my $self = shift;
+    return Scalar::Util::refaddr( $self );
+}
+
+sub BUILDARGS {
+    my ($class, $raw, @args) = @_;
+
+    warn "$class - ignoring arguments to new: @args\n"
+	if @args;
+
+    if (Elive::Util::_reftype($raw) eq 'HASH') {
+
+	my $types = $class->property_types;
+
+	my %cooked;
+
+	my $aliases = $class->_get_aliases;
+
+	foreach (keys %$raw) {
+
+	    #
+	    # apply any aliases
+	    #
+
+	    my $prop = (exists $aliases->{$_}
+			? ($aliases->{$_}{to} or die "$class has malformed alias: $_")
+			: $_);
+
+	    my $value = $raw->{$_};
+	    if (my $type = $types->{$prop}) {
+		if (ref($value)) {
+		    #
+		    # inspect the item to see if we need to uncoerce back to
+		    # a simpler type. For example we may have been passed an
+		    # object, rather than just its primary key.
+		    #
+		    my (undef, $is_array, $is_struct, $is_ref)
+			= Elive::Util::parse_type($type);
+
+		    $value = Elive::Util::string($value, $type)
+			unless $is_array || $is_struct || $is_ref;
+		}		    
+	    }
+	    else {
+		Carp::carp "$class: unknown property: $prop";
+	    }
+
+	    $cooked{$prop} = $value;
+	}
+
+	return \%cooked;
+    }
+
+    return $raw;
+}
+
 sub stringify {
     my $class = shift;
     my $data = shift;
@@ -114,6 +172,15 @@ sub entity_name {
     return $entity_class->_entity_name;
 }
 
+=head2 collection_name
+
+    my $collection_name = MyApp::Entity::User->collecion_name
+    ok($collection_name eq 'users');
+
+=cut
+
+# Class::Data::Inheritable property
+
 # _alias, _get_aliases
 #
 #    MyApp::Entity::Meeting->_set_data_mapping(requiredSeats => 'seats');
@@ -126,17 +193,16 @@ sub entity_name {
 #
 
 sub _alias {
-    my $entity_class = shift;
-    my $from = lcfirst(shift);
-    my $to = lcfirst(shift);
+    my ($entity_class, $from, $to, %opt) = @_;
 
-    die 'usage: $entity_class->_set_data_mapping(alias, prop, %opts)'
+    $from = lcfirst($from);
+    $to = lcfirst($to);
+
+    die 'usage: $entity_class->_alias(alias, prop, %opts)'
 	unless ($entity_class
 		&& $from && !ref($from)
 		&& $to && !ref($to));
 
-    my %opt = @_;
-
     my $aliases = $entity_class->_get_aliases;
 
     #
@@ -193,17 +259,32 @@ sub id {
 =cut
 
 sub primary_key {
-    my $entity_class = shift;
+    my ($entity_class, @pkey) = @_;
 
-    if (@_) {
-
-	$entity_class->_primary_key([@_]);
-
-    }
+    $entity_class->_primary_key(\@pkey)
+	if (@pkey);
 
     return @{$entity_class->_primary_key};
 }
 
+=head2 params
+
+    Setter/getter for parameter field(s) for this entity class
+
+    Elive::Entity::User->params(loginName => 'Str');
+    my %params = MyApp::Entity::User->params;
+
+=cut
+
+sub params {
+    my ($entity_class, %params) = @_;
+
+    $entity_class->_params(\%params)
+	if (keys %params);
+
+    return %{$entity_class->_params};
+}
+
 =head2 entities
 
     my $entities = Entity::Entity->entities
@@ -243,7 +324,8 @@ sub _ordered_attribute_names {
 	$order{$_} ||= ++$rank;
     }
 
-    return sort {$order{$a} <=> $order{$b}} (keys %order);
+    my @atts_sorted = sort {$order{$a} <=> $order{$b}} (keys %order);
+    return @atts_sorted;
 }
 
 sub _ordered_attributes {
@@ -255,18 +337,13 @@ sub _ordered_attributes {
 }
 
 sub _cmp_col {
+    my ($class, $data_type, $_v1, $_v2, %opt) = @_;
 
     #
     # Compare two values for a property 
     #
 
-    my $class = shift;
-    my $data_type = shift;
-    my $_v1 = shift;
-    my $_v2 = shift;
-    my %opt = @_;
-
-    return undef
+    return
 	unless (defined $_v1 && defined $_v2);
 
     my ($type, $is_array, $is_struct) = Elive::Util::parse_type($data_type);
@@ -317,7 +394,7 @@ sub _cmp_col {
 		my $v1 = $v1[$i];
 		my $v2 = $v2[$i];
 
-		if ($is_struct || $type =~ m{^(Str|Enum|HiResDate)}i) {
+		if ($is_struct || $type =~ m{^(Str|Enum|HiResDate)}ix) {
 		    #
 		    # string comparision. works on simple strings and
 		    # stringified entities. Also used for hires dates
@@ -327,19 +404,19 @@ sub _cmp_col {
 			      ? uc($v1) cmp uc($v2)
 			      : $v1 cmp $v2);
 		}
-		elsif ($type =~ m{^Bool}i) {
+		elsif ($type =~ m{^Bool}ix) {
 		    # boolean comparison
 		    $cmp ||= ($v1 eq 'true'? 1: 0) <=> ($v2 eq 'true'? 1: 0);
 		}
-		elsif ($type =~ m{^Int}i) {
+		elsif ($type =~ m{^Int}ix) {
 		    # int comparision
 		    $cmp ||= $v1 <=> $v2;
 		}
-		elsif ($type =~ m{^Ref}i) {
+		elsif ($type =~ m{^Ref}ix) {
 		    $cmp ||= YAML::Dump($v1) cmp YAML::Dump($v2);
 		}
 		else {
-		    die "class $class: unknown type: $type";
+		    Carp::croak "class $class: unknown type: $type\n";
 		}
 	    }
 	}
@@ -409,8 +486,7 @@ Assign values to entity properties.
 =cut
 
 sub set {
-    my $self = shift;
-    my %data = @_;
+    my ($self, %data) = @_;
 
     my %entity_column = map {$_ => 1} ($self->properties);
     my %primary_key = map {$_ => 1} ($self->primary_key);
@@ -418,10 +494,13 @@ sub set {
     foreach (keys %data) {
 
 	unless ($entity_column{$_}) {
-	    warn ((ref($self)||$self).": unknown property: $_");
+	    Carp::carp ((ref($self)||$self).": unknown property: $_");
 	    next;
 	}
 
+	my $type = $self->property_types->{$_}
+	   or die ((ref($self)||$self).": unable to determine property type for field: $_");
+
 	if (exists $primary_key{ $_ }) {
 
 	    my $old_val = $self->{$_};
@@ -429,7 +508,7 @@ sub set {
 	    if (defined $old_val && !defined $data{$_}) {
 		die "attempt to delete primary key";
                }
-	    elsif ($self->_cmp_col($self->property_types->{$_},
+	    elsif ($self->_cmp_col($type,
 				   $old_val, $data{$_})) {
 		die "attempt to update primary key";
 	    }
@@ -441,8 +520,20 @@ sub set {
 
 	if (defined $value) {
 
-	    $self->$_($value);
+	    if (ref($value)) {
+		#
+		# inspect the item to see if we need to uncoerce back to
+		# a simpler type. For example we may have been passed an
+		# object, rather than just its primary key.
+		#
+		my (undef, $is_array, $is_struct, $is_ref)
+		    = Elive::Util::parse_type($type);
+		
+		$value = Elive::Util::string($value, $type)
+		    unless $is_array || $is_struct || $is_ref;
+	    }
 
+	    $self->$_($value);
 	}
 	else {
 
@@ -456,4 +547,45 @@ sub set {
     return $self;
 }
 
+sub can {
+    my ($class, $method) = @_;
+
+    my $subref;
+
+    unless ($subref = eval{ $class->SUPER::can($method) }) {
+
+	my $aliases = eval{ $class->_aliases };
+
+	if ($aliases && $aliases->{$method}
+	    && (my $alias_to = $aliases->{$method}{to})) {
+	    $subref =  $class->SUPER::can($alias_to);
+	}
+    }
+
+    return $subref;
+}
+
+sub AUTOLOAD {
+    my @class_path = split('::', ${Elive::Struct::AUTOLOAD});
+
+    my $method = pop(@class_path);
+    my $class = join('::', @class_path);
+
+    die "Autoload Dispatch Error: ".${Elive::Struct::AUTOLOAD}
+        unless $class && $method;
+
+    if (my $subref = $class->can($method)) {
+
+	{
+	    no strict 'refs';
+	    *{$class.'::'.$method} = $subref;
+	}
+
+	goto $subref;
+    }
+    else {
+	Carp::croak $class.": unknown method $method";
+    }
+}
+
 1;
